use crate::RenderError;

use crate::{Buffer, Escape, EscapeHtml, EscapeJsonString, RenderOnce};

pub use crate::filter;

#[inline(always)]
pub fn esc_html() -> EscapeHtml {
    EscapeHtml
}

#[inline(always)]
pub fn esc_json() -> EscapeJsonString {
    EscapeJsonString
}

#[inline(always)]
pub fn render<T: RenderOnce>(buf: &mut Buffer, value: T) -> Result<(), RenderError> {
    value.render_once(buf)
}

#[inline(always)]
pub fn render_escaped<T: RenderOnce, E: Escape>(
    buf: &mut Buffer,
    value: T,
    escape: &E,
) -> Result<(), RenderError> {
    value.render_once_escaped(buf, escape)
}

#[inline(always)]
pub fn render_text(buf: &mut Buffer, value: &str) {
    buf.push_str(value)
}
